function  [H,P,ARL_params] = CRFB2_data_by_sbj(D,out,P)

%CRFB2_data_by_sbj -- Preprocess the data of the CRFeedback2 experiment.
%
% H = CRFB2_data_by_sbj(D)
% [H,ARL_params,P] = CRFB2_data_by_sbj(D,out,P)
%
% D is a data structure constructed by CRFB2_import_data from the raw
% data files. The field 'data' is particularly relevant.
% The result H is a structure with one entry for each entry in D and
% the following fields:
%  - sbj     : subject number
%  - group   : group number
%  - dry_run -- 0=HUMAN_DATA, 1+=xxxx_DRY_RUN. See P.enum.dry_run_enum
%============== CRFB2_data_by_sbj1 fields
%  - MOTION_data  : a [350x9] matrix for the MOTION trials
%  - TEXTURE_data : a [350x9] matrix for the TEXTURE trials, with fields:
%    1=block      : 1-13
%    2=blk_trial  : Trial number in current block
%    3=schedule   : 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%    4=categ      : 1 through 7. This is also the correct answer.
%    5=orient     : degrees from vertical
%    6=feedbackp  : 1=feedback, 0=no_feedback
%    7=response   : integer from 1 to N_categ
%    8=correct    : 0=incorrect=(resp~=categ) or 1=correct=(resp==categ)
%    9=RT         : from stim onset until the (valid) keypress, msec
%  - schedule28   : [13x1] 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%  - feedback28   : [13x1] 1=feedback, 0=no_feedback
%  - MOTION_ARL28 : [13x1] ARLs calculatd in 28-trial blocks, MOTION task
%  - TEXTURE_ARL28: [13x1] ARLs calculatd in 28-trial blocks, TEXTURE task
%  - schedule42   : [9x1] 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%  - feedback42   : [9x1] 1=feedback, 0=no_feedback
%  - MOTION_ARL42 : [9x1] ARLs calculatd in 42-trial blocks, MOTION task
%  - TEXTURE_ARL42: [9x1] ARLs calculatd in 42-trial blocks, TEXTURE task
%============== Post-processing
%  No post-processing seems necessary
%
% The first ARL in each sequence is calculated only from the 14 UNIF trials
% of the corresponding task. It is best to ignore it in the anlyses.
%
% Set OUT to [] to suppress reporting to stdout. Default=1.
% P is the master parameter struct for CRFeedback2. Default P=CRFB2_params(0).
%
% Example: (Assuming all data are in a single file 'raw_data.dat')
%   D = CRFB2_import_data ; H = CRFB2_data_by_sbj(D) ;
%
% See also CRFB2_data_by_sbj1, CRFB2_avg_resp_level, CRFB2_import_data, CRFB2_empir_ARL_profile.

% (c) Laboratory for Cognitive Modeling and Computational Cognitive
% Neuroscience at the Ohio State University, http://cogmod.osu.edu
%
% 1.0.0 2009-12-04 AP: Wrote it, based on CMUExper/fdbk1/data/data_by_sbj.m

if (nargin<2) ; out = 1 ; end   % report to stdout by default
if (nargin < 3) ; P = CRFB2_params(0) ; end

%-- Call CRFB2_avg_resp_level to generate ARL_params for use in the loop
ARL_params = CRFB2_avg_resp_level(P) ;

%-- Main loop
for k = 1:length(D)
  H(k).sbj = D(k).sbj ;       %#ok<*AGROW>
  H(k).group = D(k).group ;
  H(k).dry_run = D(k).dry_run ;
  data = D(k).data ;
  data(D(k).skip_idx,:) = [] ;
  fprintN(out,'Processing %5d trials for participant %3d... ',...
          size(data,1),D(k).sbj) ;

  %- CRFB2_hitcount1 does all the work
  H1 = CRFB2_data_by_sbj1(data,ARL_params,P) ;  % <-- Sic!
  
  %- Copy all field names from H1 to H(k)
  H(k).MOTION_data = H1.MOTION_data ;
  H(k).TEXTURE_data = H1.TEXTURE_data ;
  H(k).schedule28 = H1.schedule28 ;
  H(k).feedback28 = H1.feedback28 ;
  H(k).MOTION_ARL28 = H1.MOTION_ARL28 ;
  H(k).TEXTURE_ARL28 = H1.TEXTURE_ARL28 ;
  H(k).schedule42 = H1.schedule42 ;
  H(k).feedback42 = H1.feedback42 ;
  H(k).MOTION_ARL42 = H1.MOTION_ARL42 ;
  H(k).TEXTURE_ARL42 = H1.TEXTURE_ARL42 ;

  fprintN(out,'Done.\n') ;
end

%---  Return H
%%%%% End of CRFB2_hitcount.M
