function  S = data_by_sbj(sbj,group,dist,categ,fdbk,OK,resp,RT)

% DATA_BY_SBJ -- Organize FDBK1 data in a struct array with 1 elt per sbj.
%
% S = data_by_sbj(sbj,group,dist,categ,fdbk,OK,resp,RT)
%
%  The arguments to the function are long column vectors, each keeping
%  an individual variable as read from the raw data file (see IMPORT_DATA).
%  All column vectors must have the same length. When arranged in a 
%  rectangular matrix, each row represents a trial in the experiment.
%
%  The function rearranges the data into a structure array S organized by 
%  subject. Each structure has the following fields:
%    1. sbj_number -- subject number; scalar ranging 1:99
%    2. group -- the group this subject belongs to: 1=U, 2=P, 3=N
%    3. miss_idx -- indices of the trials with invalid (~=1:7) keys
%    4. dist -- physical distance b/n the two dots; 476x1 ordered by trial
%    5. categ -- category label: 420=1,460=2,...,660=7 ; 476x1 vector
%    6. fdbk -- feedback: 1:7 or 0 (=no feedback)
%    7. resp -- responses of the subject (0==bad key) ; 476x1 vector
%    8. RT -- raw response times in msec; 476x1 vector
%    9. interv -- number of intervening trials since the most recent
%         trial with identical stimulus (see function PRIOR_REPETITION)
%   10. ARL -- average response level (window width = 28); 17x1 vector
%
%  See also IMPORT_DATA, FDBK_ANALYSIS, PRIOR_REPETITION, AVG_RESP_LEVEL.

%  Original coding by Alexander Petrov, Carnegie Mellon University.
%  $Revision: 1.0 $  $Date: 2001/04/24 15:00 $

%-- Separate the data for each subject
data = [group,dist,categ,fdbk,OK,resp,RT] ;  % pack into one big matrix
[split_data,sbj_n] = split(data,sbj) ;       % split into small matrices
number_of_subjects = length(sbj_n) ;

%-- Control parameters for AVG_RESP_LEVEL
ARL_params = struct('data_format',2, 'midrange_stim',540, 'cutoff',2.0) ;
ARL_params.window_width = 28 ;
ARL_params.endpoints = [28:28:476] ;


%-- Main loop
for k = 1:number_of_subjects
   s = sbj_n(k) ;          % the sbj-number of this subject
   d = split_data{k} ;
   %- Pull out individual variables; each one typically is a 450x1 vector
   group1 = d(:,1) ;
   dist1  = d(:,2) ;
   categ1 = d(:,3) ;
   fdbk1  = d(:,4) ;
   OK1    = d(:,5) ;
   resp1  = d(:,6) ;
   RT1    = min(d(:,7),10000) ;    % clip outliers
   %- Number of intervening trials since last repetition
   [foo,interv1] = prior_repetition(dist1) ;
   interv1 = interv1 - 1 ;
   %- GROUP should be constant across all trials for any given subject
   if (min(group1) ~= max(group1)) 
     warning('The GROUP of subject %d is not constant across trials.',s) ;
   end
   group1 = min(group1) ;     % scalar
   %- Handle missing values
   miss_idx = find(OK1==0) ;
   %- Build up the structure
   S(k).sbj_number = s ;         % 1.
   S(k).group = group1 ;         % 2.
   S(k).miss_idx = miss_idx ;    % 3.
   S(k).dist = dist1 ;           % 4.
   S(k).categ = categ1 ;         % 5.
   S(k).fdbk = fdbk1 ;           % 6.
   S(k).resp = resp1 ;           % 7.
   S(k).RT = RT1 ;               % 8.
   S(k).interv = interv1 ;       % 9.
   %- Pass it to AVG_RESP_LEVEL and fill in the last field
   ARL = avg_resp_level(S(k),ARL_params)' ;
   S(k).ARL = ARL ;              % 10.
end

% Return S
%%%%% End of DATA_BY_SBJ.M
