function  ARL = fdbk1_ARL(stim,resp,ARL_params)

% FDBK1_ARL -- Calculate Average Response Levels (ARLs) for FDBK1 Experiment.
%   
% ARL = fdbk1_ARL(stim,resp)
% ARL = fdbk1_ARL(stim,resp,ARL_params)
% [ARL,ARL_params] = fdbk1_ARL(...)
%
% STIM and RESP are 476-by-Nruns matrices. ARL is 17-by-Nruns.
% ARL_PARAMS defaults to struct('window_width',28,'endpoints',[28:28:476],
%   ... 'midrange_stim',540,'cutoff',2.0)
% 
% Does not handle missing values. Use CR_SLIDE_REGRESS for that.
%
% Sanity check:  Compare with empirical data for a few Ss with miss_idx=[]
%   load('S.mat');idx=[1 3 5];stim=[S(idx).dist];resp=[S(idx).resp];
%   ARL=fdbk1_ARL(stim,resp);max(abs([S(idx).ARL]-ARL))
%
% See also AVG_RESP_LEVEL, DATA_BY_SBJ, FDBK1_ARL2, CR_SLIDE_REGRESS.

% Original coding by Alex Petrov, Ohio State University
% $Revision: 1.0 $  $Date: 2007-06-04 $

if (nargin<3)  ARL_params = [] ; end
if (isempty(ARL_params))
    ARL_params = struct('window_width',28,'endpoints',[28:28:476],...
                        'midrange_stim',540,'cutoff',2.0) ;
end

%-- Decipher params
win_width = ARL_params.window_width ;
endpoints = ARL_params.endpoints ;
mid_stim = ARL_params.midrange_stim ;
cutoff = ARL_params.cutoff ;
N_trials = endpoints(end) ;    % 476=17*28

[k,N_runs] = size(stim) ;
if (k~=N_trials)
    warning('Non-standard sequence length: %d instead of %d.',k,N_trials) ;
end

%--- Prepare the BLOCKS matrix. 
% See CMUExper/CatRat1/data/trend/avg_resp_level.m
N_blocks = length(endpoints) ;
initpoints = endpoints - win_width ;
blocks = repmat([1:win_width]',1,N_blocks) + repmat(initpoints,win_width,1) ;
tr = [1:N_trials]' ;
for b = 1:N_blocks
    idx{b} = find(ismember(tr,blocks(:,b))) ;
end

%--- Main loop: Do the sliding regressions and calculate the ARLs
% See CMUExper/CatRat1/data/trend/CR_slide_regress.m
ARL = zeros(N_blocks,N_runs) ;
for k = 1:N_runs
  for b = 1:N_blocks
    coefs = regcoefs(stim(idx{b},k),resp(idx{b},k),cutoff) ; % <-- main work
    ARL(b,k) = coefs(2) * mid_stim + coefs(1) ;    % slope * X + intercept
  end  
end

%---  Return ARL and possibly ARL_PARAMS
%%%%% End of file FDBK1_ARL.M
