%% can_INST_compensate.m
%
% Are there parameter regimes in which the instance-based model "INST"
% (implemented in work/models/protoanchor.m) exhibits less assimilation
% without feedback that it does with feedback under skewed stimulus
% distributions?
%
% The tendency off all responses to drift upwards without feedback is
% eliminated in order to see the context-induced effects more clearly.
%
%  File: work/models/protoanchor/can_INST_compensate.m
%  Date: 2007-06-05
%  Alexander Petrov, http://alexpetrov.com


%% Make a test bank of stimulus sequences
%
% * Gr1 -- Uniform (not skewed), feedback first. Should be no assimilation
% * Gr2 -- Positively skewed, feedback first. Assimilation decreases ARL
% * Gr3 -- Negatively skewed, feedback first. Assimilation increases ARL
% * Gr4 -- Uniform (not skewed), feedback second. Should be no assimilation
% * Gr5 -- Positively skewed, feedback second. Assimilation decreases ARL
% * Gr6 -- Negatively skewed, feedback second. Assimilation increases ARL
clear all
cd(fullfile(work_pathstr,'models','protoanchor')) ;
N_runs_per_group = 100 ;
groups = [2 3  5 6] ;
fdbk_first_p = [1 1 1 0 0 0] ;    % exp schedule by group
gr_descr = {'Ufb','Pfb','Nfb','xxx','Pno','Nno' } ;
for k = groups
    [stim{k},categ{k},fdbk{k}] = make_fdbk1_stimuli(k,N_runs_per_group) ;
end

%% Parameters for the INST model
H = 0.040 ;
T = 0.040 ;
d = 0.50 ;
c = 0.80 ;        % in category-size units
cutoffs = [-3 -1 +1 +3].*c ;   % eliminate drift

iP = protoanchor_params([1:7]) ;  % instance-based model (aka "protoanchor")
iP.history = H ;
iP.temper = T ;
iP.decay = d ;
iP.cutoffs = cutoffs
%iP.instances

%% Run the model and calculate ARL profiles
N_blocks = 17 ;
iARL_profile = zeros(N_blocks,max(groups)) ;
for k = groups
    imresp{k} = protoanchor(stim{k},iP,fdbk{k}) ;
    iARL{k} = fdbk1_ARL(stim{k},imresp{k}) ;
    iARL_profile(:,k) = mean(iARL{k},2) ;
end

%% Plot the group ARLs for the INST model
% See CMUExper/fdbk1/data/model_ARL_scplot.m
ls = {'ks-' 'b>-' 'r<-' 'ks--' 'b>--' 'r<--'} ;   % line styles
x=[14:28:476]';
ax = [0 500 3.4 4.6] ;
clf ; hold on ;
for k = groups
    plot(x,iARL_profile(:,k),ls{k}) ;
end
hold off;box on;
axis(ax);set(gca,'xtick',[0 [28:112:476]]);grid on;
xlabel('Trial');ylabel('Average response level');
title(sprintf('INST: H=%5.3f, T=%5.3f, d=%4.2f, c=%s, %d runs',...
      iP.history,iP.temper,iP.decay,mat2str(iP.cutoffs),N_runs_per_group));
hh=refline(0,4);set(hh,'color','k','linestyle','-');

%% Define Assimilation := ARL(N)-ARL(P)
% See CMUExper/fdbk1/data/assimil_profiles.m
%
% * 1 = Feedback first = Group 3 - Group 2
% * 2 = No-feedback first = Group 6 - Group 5
%
iAssim = iARL_profile(:,[3 6]) - iARL_profile(:,[2 5]) ;
hh = plot(x,iAssim(:,1),'ro-',x,iAssim(:,2),'bs--') ;
set(hh,'LineWidth',2) ;
ax = [0 500 -0.5 1.5] ; axis(ax) ;
set(gca,'xtick',[0 [28:112:476]]);xlabel('Trial');grid on;
ylabel('Assimilation = ARL(N)-ARL(P)');
title(sprintf('INST: H=%5.3f, T=%5.3f, d=%4.2f, c=%s, %d runs',...
      iP.history,iP.temper,iP.decay,mat2str(iP.cutoffs),N_runs_per_group));
hh=refline(0,0);set(hh,'color','k','linestyle','-');
legend('Feedback first','No-feedback first',4) ;

%% Collapse across blocks and print report
iARL_fdbk   = zeros(N_runs_per_group,max(groups)) ;
iARL_nofdbk = zeros(N_runs_per_group,max(groups)) ;
for k = groups
    if (fdbk_first_p(k))
       iARL_fdbk(:,k) = mean(iARL{k}([2:5 10:13],:))' ;
       iARL_nofdbk(:,k) = mean(iARL{k}([6:9 14:17],:))' ;
    else
       iARL_fdbk(:,k) = mean(iARL{k}([6:9 14:17],:))' ;
       iARL_nofdbk(:,k) = mean(iARL{k}([2:5 10:13],:))' ;
    end
end
mn_iARL_fdbk = mean(iARL_fdbk) ;
mn_iARL_nofdbk = mean(iARL_nofdbk) ;
mn_iAssim_fdbk = mn_iARL_fdbk([3 6]) - mn_iARL_fdbk([2 5]) ;
mn_iAssim_nofdbk = mn_iARL_nofdbk([3 6]) - mn_iARL_nofdbk([2 5]) ;

fid = fopen('can_INST_compensate.log','a') ;
s = [1 fid] ;  % print both on stdout and into file
fprintN(s,'\nParams: H=%5.3f, T=%5.3f, d=%4.2f, c=%s, %3d runs',...
        iP.history,iP.temper,iP.decay,mat2str(iP.cutoffs),N_runs_per_group);
fprintN(s,'\n-----------------------------------------------------------------') ;
fprintN(s,'\nRed line:  fdbk1 = %+5.2f, nofb1 = %+5.2f, f-n = %+5.2f   %% fdbk 1st',...
        mn_iAssim_fdbk(1),mn_iAssim_nofdbk(1),...
        mn_iAssim_fdbk(1)-mn_iAssim_nofdbk(1)) ;
fprintN(s,'\nBlue line: nofb2 = %+5.2f, fdbk2 = %+5.2f, f-n = %+5.2f   %% nofb 1st',...
        mn_iAssim_nofdbk(2),mn_iAssim_fdbk(2),...
        mn_iAssim_fdbk(2)-mn_iAssim_nofdbk(2)) ;
fprintN(s,'\nInteractn: first = %+5.2f, secnd = %+5.2f, TTL = %+5.2f\n',...
        mn_iAssim_fdbk(1)-mn_iAssim_nofdbk(2),...     % blocks 2:5 and 10:13
        mn_iAssim_fdbk(2)-mn_iAssim_nofdbk(1),...     % blocks 6:9 and 14:17
        sum(mn_iAssim_fdbk)-sum(mn_iAssim_nofdbk)) ;
fclose (fid) ;

%% Scatterplot for the Negative groups 3 & 6. INST model
% For Negatively skewed stimuli, higher ARLs indicate
% assimilation and lower ARLs indicate compensation.
ref4n = [4 4 9 ; 0 4 9]' ;
ax = [3 5 3 5] ;
idx = [1:min(120,N_runs_per_group)]' ;   % avoid overly dense scatterplots
clf;hold on
hh=plot(iARL_fdbk(idx,3),iARL_nofdbk(idx,3),'r.',...
        iARL_fdbk(idx,6),iARL_nofdbk(idx,6),'b.') ;  % data by sbj
%set(hh,'LineWidth',2) ;
hh=plot(mn_iARL_fdbk(3),mn_iARL_nofdbk(3),'ko',...
        mn_iARL_fdbk(6),mn_iARL_nofdbk(6),'ks') ;   % group means
set(hh,'LineWidth',2,'MarkerSize',12) ;
plot(ref4n(:,1),ref4n(:,2),'k.-') ;
hold off ; box on ;
axis(ax) ; axis square ; grid on ;
xlabel('ARL with feedback') ; ylabel('ARL without feedback') ;
title(sprintf('INST: Neg. skew. H=%5.3f, T=%5.3f, d=%4.2f',...
      iP.history,iP.temper,iP.decay)) ;
legend('Fdbk 1st  (Gr 3)','Fdbk 2nd (Gr 6)','Location','NorthWest') ;

%% Scatterplot for the Positive groups 2 & 5. INST model
% For Negatively skewed stimuli, higher ARLs indicate
% assimilation and lower ARLs indicate compensation.
ref4p = [0 4 4 ; 0 4 9]' ;
clf;hold on
hh=plot(iARL_fdbk(idx,2),iARL_nofdbk(idx,2),'r.',...
        iARL_fdbk(idx,5),iARL_nofdbk(idx,5),'b.') ;  % data by sbj
%set(hh,'LineWidth',2) ;
hh=plot(mn_iARL_fdbk(2),mn_iARL_nofdbk(2),'ko',...
        mn_iARL_fdbk(5),mn_iARL_nofdbk(5),'ks') ;   % group means
set(hh,'LineWidth',2,'MarkerSize',12) ;
plot(ref4p(:,1),ref4p(:,2),'k.-') ;
hold off ; box on ;
axis(ax) ; axis square ; grid on ;
xlabel('ARL with feedback') ; ylabel('ARL without feedback') ;
title(sprintf('INST: Pos. skew. H=%5.3f, T=%5.3f, d=%4.2f',...
      iP.history,iP.temper,iP.decay)) ;
legend('Fdbk 1st  (Gr 2)','Fdbk 2nd (Gr 5)','Location','NorthWest') ;

