function  responses = protoanchor(arg1,params,fdbk)

% PROTOANCHOR  -- Instance-based variant of the ANCHOR cognitive model.
%   
%  responses = protoanchor(data)
%  responses = protoanchor(stimuli,params)
%  responses = protoanchor(stimuli,params,fdbk)
%
%  Each trial leaves a separate instance: <magnitude,label,timestamp>
%
%  Instead of supplying two (or three) input arguments, the same
%  information can be passed in a single structure with fields
%  'stim', 'params', and optionally 'fdbk'. If the first
%  argument is a structure, no other arguments can be passed.
%
%  See also ANCHOR2, PROTOANCHOR_LOG, PROTOANCHOR_PARAMS.

% Original coding by Alex Petrov, Ohio State University
% $Revision: 1.0 $  $Date: 2007-04-06 16:16 $
%
% Copyright (c) Alexander Petrov 1999-2007, http://alexpetrov.com
% Please read the LICENSE and NO WARRANTY statement in anchor_license.m

%---  Handle arguments
switch nargin
  case 1               % a single structure with subfields
    if (isstruct(arg1))
      stimuli = arg1.stim ;
      params = arg1.params ;
      if (isfield(arg1,'fdbk'))   % feedback field supplied?
        fdbk = arg1.fdbk ;
      else
        fdbk = [] ;               % default: do not use feedback
      end
    else
      error('When there is only one input arg, it must be a structure.') ;
    end
  case 2
    % STIMULI and PARAMS explicitly supplied
    stimuli = arg1 ;
    fdbk = [] ;       % default
  case 3
    stimuli = arg1 ;  % all three pieces of information explicitly supplied
  otherwise
    error('Incorrect number of input arguments.') ;
end

[N_trials,N_sbj] = size(stimuli) ;

if (isempty(fdbk))
  fdbk = zeros(N_trials,N_sbj) ;   % no feedback
elseif (any(size(fdbk)~=[N_trials,N_sbj]))
  error('STIMULI and FEEDBACK must have the same shape.') ;
end

%---  Decipher PARAMS
inter_trial_interval = params.ITI ;
SM_conv = params.SM_conv ;     % stimulus continuum --> magnitude continuum
N_cat = params.N_cat ;         % number of categories == number of anchors
category_sz = params.cat_sz ;  % width of one response category
cutoffs = params.cutoffs * category_sz ;  % [-2 -1 +1 +2]
max_incrmt = length(cutoffs) / 2 ;
perc_k = params.perc_k ;       % noise in the target magnitude
mem_k = params.mem_k ;         % noise in anchor magnitudes
temper = params.temper ;       % temperature for anchor softmax
H = params.history ;           % history factor (=1/mismatch_penalty)
minus_decay = -params.decay ;  % activation decay rate
%alpha = params.alpha ;         % competitive learning rate
%one_alpha = 1 - alpha ;
N_seed_inst = size(params.instances,1) ;   % number of seed instances

%---  Convert physical stimuli into internal magnitudes ; avoid MAG_SCALE
%  Only LINEAR scales are supported, that is, Stevens law with n=1.
mags = SM_conv .* stimuli ;                            % Stevens law
mags = mags .* normrnd(ones(N_trials,N_sbj),perc_k) ;  % Ekman's law

%---  Main work
responses = zeros(N_trials,N_sbj) ;
for k = 1:N_sbj
  %-- Re-initialize
  inst_mags = [params.instances(:,1) ; zeros(N_trials,1) ] ;
  inst_resps = [params.instances(:,2) ; zeros(N_trials,1) ] ;
  time_of_creation = [params.instances(:,3) ; zeros(N_trials,1) ] ;
  M = N_seed_inst ;  % current number of active entries in INST_xxxx
  curr_time = 0 ;
  %-- Main loop
  for tr = 1:N_trials
    target_mag = mags(tr,k) ;
    curr_time = curr_time + inter_trial_interval ;
    %- Calculate the base-level activations of the anchors
    T = curr_time - time_of_creation(1:M) ;  % lifetimes of the instances
    B = minus_decay .* log(T) ;      % base-level activation
    %- Add multiplicative noise to the anchor magnitudes
    noisy_inst_mags = inst_mags(1:M) .* normrnd(ones(M,1),mem_k) ;
    %- Form match scores:  Si = -|M-Ai| + H*Bi
    mismatch = abs(noisy_inst_mags - target_mag) ;
    scores = H.*B - mismatch ;
    %- Retrieve an instance according to the Boltzmann equation
    scores = scores - min(scores) ;  % make all scores have the same sign
    [win_score,curr_inst_idx] = softmax1(scores,temper) ;
    curr_inst_resp = inst_resps(curr_inst_idx) ;
    inst_mag = noisy_inst_mags(curr_inst_idx) ;
    %- Determine the correction
    delta = target_mag - inst_mag ;
    curr_incrmt = max_incrmt - sum(cutoffs > delta) ;
    %- Correct the instance response by the incrmt; incrmt can be negative
    target_resp = curr_inst_resp + curr_incrmt ;
    target_resp = max(1,min(N_cat,target_resp)) ;  % clip within bounds
    %- Register response
    responses(tr,k) = target_resp ;
    %- Store the instance, with or without feedback
    stored_resp = fdbk(tr,k) ;
    if (stored_resp==0)             % if no external feedback
      stored_resp = target_resp ;   % rehearse own response
    end
    %fprintf('trial %d, storing instance %d.\n',tr,stored_resp) ;
    M = M + 1 ;
    inst_mags(M) = target_mag ;
    inst_resps(M) = stored_resp ;
    time_of_creation(M) = curr_time ;
  end % of tr=1:N_trials
end % of k=1:N_sbj

%--- Return RESPONSES
%%%%% End of file ANCHOR2.M
