function  search_params = protoanchor_search_params(spec,fminconp)

% PROTOANCHOR_SEARCH_PARAMS -- Parameters for PROTOANCHOR parameter search.
%   
%  search_params = protoanchor_search_params
%  search_params = protoanchor_search_params(seed)
%  search_params = protoanchor_search_params(flags)
%  search_params = protoanchor_search_params(flags,fminconp)
%
%  When supplied, SEED is structure that is filled in with
%  default values for the omitted slots. Existing values are preserved.
%  Alternatively, an eight-element FLAGS vector can be supplied. It is 
%  passed to ANCHOR_TEMPLATE. Default FLAGS=[0 1 1 0 0 0 0 0].
%  When providing explicit conversion templates, it is up to the caller
%  to ensure that P2V_TEMPL, V2P_TEMPL, and BOUNDS are consistent. It is
%  therefore a bad idea to supply any one of these three slots without
%  supplying the other two as well. (Any missing templates or bounds will
%  be filled in with defaults as defined in ANCHOR_TEMPLATE.)
%  When FMINCONP = 1 (the default), the search parameters are tailored
%  for the FMINCON optimizer. Otherwise they are tailored for LSQNONLIN.
%
%  Experiments fitting synthetic ARL profiles showed that FMINCON tends
%  to drive the error lower than does LSQNONLIN. The latter tends to
%  reduce the error more quickly at first, but then bogs down and
%  eventually gives up. Apparently, some of its assumptions are violated.
%  (LSQNONLIN uses a 'Large Scale' algorithm whereas FMINCON uses a
%  'Medium Scale' algorithm. See Optimization Toolbox documentation.)
%
%  Example: Sparams = protoanchor_search_params
%    model_name: 'protoanchor_ARL2' % model M-file (see FDBK1_ARL2)
%        params: [1x1 struct]   % modified version of PROTOANCHOR_PARAMS
%     p2v_templ: {1x2 cell}     % see PARAMS2VECTOR and ANCHOR_TEMPLATE
%     v2p_templ: {1x2 cell}     % see VECTOR2PARAMS and ANCHOR_TEMPLATE
%        bounds: [2x2 double]   % upper and lower bounds for each dimension
%    ARL_params: [1x1 struct]   % for FDBK1_ARL2 in PROTOANCHOR_ARL2
%   funfun_name: 'fmincon'      % optimization engine
%         optns: [1x1 struct]   % optimization options: optimset('fmincon')
%    lsfun_name: 'sumsqerr'     % loss function = sum squared error
%   target_vals: []             % MUST BE SET MANUALLY LATER! 
%
% See also PROTOANCHOR_ARL2, PARAMSEARCH, ANCHOR_SEARCH_PARAMS2, ANCHOR_TEMPLATE.

% Original coding by Alex Petrov, Ohio State University
% $Revision: 1.0 $  $Date: 2007-08-16 $
%
% Copyright (c) Alexander Petrov 1999-2007, http://alexpetrov.com
% Please read the LICENSE and NO WARRANTY statement in anchor_license.m

if (nargin==0)                % default
   search_params = struct('model_name','protoanchor_ARL2') ;
   flags = [0 1 1 0 0 0 0 0] ;
elseif (isstruct(spec))       % seed parameter structure
   search_params = spec ;
   flags = [0 1 1 0 0 0 0 0] ;
elseif (isnumeric(spec))      % FLAGS vector
   flags = spec ;
   search_params = struct('model_name','protoanchor_ARL2') ;
else
   error('SPEC must be either seed structure or flag vector.') ;
end
if (nargin<2) ; fminconp = 1 ; end


%--- Fill-in missing slots
if (~isfield(search_params,'model_name'))
  search_params.model_name = 'protoanchor_ARL2' ;
end
if (~isfield(search_params,'params'))
  Mparams = protoanchor_params([1:7]) ;
  Mparams.history = .040 ; 
  search_params.params = Mparams ;
end
[p2v_templ,v2p_templ,bounds] = anchor_template(flags) ;
if (~isfield(search_params,'p2v_templ'))
  search_params.p2v_templ = p2v_templ ;
end
if (~isfield(search_params,'v2p_templ'))
  search_params.v2p_templ = v2p_templ ;
end
if (~isfield(search_params,'bounds'))
  search_params.bounds = bounds ;
end
if (~isfield(search_params,'ARL_params'))  % see FDBK1_ARL2
    ARL_params = struct('window_width',56,'endpoints',[28:56:476],...
                        'midrange_stim',540,'cutoff',2.0) ;
    search_params.ARL_params = ARL_params ;
end
if (~isfield(search_params,'funfun_name'))
    if (fminconp)
        search_params.funfun_name = 'fmincon' ;
    else
        search_params.funfun_name = 'lsqnonlin' ;
    end
end
if (~isfield(search_params,'optns'))
  optns = optimset(search_params.funfun_name) ;
  optns = optimset(optns,'TolFun',1e-4,'TolX',1e-4,... 
    'MaxIter',40,'MaxFunEvals',200,'Display','iter') ;
  if (fminconp)
      optns = optimset(optns,'LargeScale','off','TolCon',1e-4) ;
  end
  search_params.optns = optns ;
end
if (~isfield(search_params,'lsfun_name'))
    if (fminconp)
        search_params.lsfun_name = 'sumsqerr' ;
    else
        search_params.lsfun_name = 'curvediff' ;
    end
end
if (~isfield(search_params,'target_vals'))
  search_params.target_vals = [] ;      % MUST BE SET MANUALLY LATER!
end

%--- Return SEARCH_PARAMS
%%%%% End of file PROTOANCHOR_SEARCH_PARAMS.M
